/*
* Delivered as a contribution to Web Services Interoperability
* Organization (WS-I) under Section (2) of the WS-I IPR Agreement.
*
* Copyright (c) 2003 Sun Microsystems, Inc.
* All rights reserved. 
*/

package com.sun.wsi.scm.util;

import java.util.Vector;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Properties;

import java.io.File;
import java.io.InputStream;
import java.io.IOException;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.sql.ResultSet;

import com.sun.wsi.scm.logging.LogEntry;

import javax.xml.rpc.JAXRPCException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.sun.wsi.scm.util.WSIConstants;

import javax.xml.rpc.server.ServletEndpointContext;
import javax.servlet.ServletContext;

public class DBConnectionPool implements WSIConstants {
	String CREATE_LOG = "CREATE TABLE log (DemoUserID VARCHAR(255), ServiceID VARCHAR(255), EventID VARCHAR(255), EventDescription VARCHAR(255), Timestamp TIMESTAMP)";
	String INSERT_LOG = "INSERT INTO log (DemoUserID, ServiceID, EventID, EventDescription, Timestamp) VALUES(?, ?, ?, ?, ?)";
	String QUERY_LOG = "SELECT DemoUserID, ServiceID, EventID, EventDescription, Timestamp from log where DemoUserID=?";

	String CREATE_CALLBACK = "CREATE TABLE callback (conversationID VARCHAR(255), warehouse_role VARCHAR(255), manufacturer_role VARCHAR(255))";
	String INSERT_CALLBACK = "INSERT INTO callback (conversationID, warehouse_role, manufacturer_role) VALUES(?, ?, ?)";
	String QUERY_CALLBACK = "SELECT warehouse_role, manufacturer_role from callback where conversationID=?";

	String dbDriver = "";
	String dbUrl = "";
	String dbUsername = "";
	String dbPassword = "";
	int dbTableExist = 0;

	private static DBConnectionPool thePool = null;

	PreparedStatement loggingInsertStatement = null;
	PreparedStatement loggingQueryStatement = null;

	PreparedStatement callbackInsertStatement = null;
	PreparedStatement callbackQueryStatement = null;

	public static int LOGGING = 0;
	public static int CALLBACK = 1;

	Connection connection = null;
	Log logger = null;

	static ServletContext servletContext = null;

	private DBConnectionPool(ServletContext context) {
		LogFactory logFactory = LogFactory.getFactory();
		logger = logFactory.getInstance(LOGGER);

		Statement statement = null;

		try {
			try {
				InputStream is = servletContext.getResourceAsStream("/WEB-INF/db.props");
				Properties dbProps = new Properties();
				dbProps.load(is);
				dbDriver = dbProps.getProperty("db.driver");
				dbUrl = dbProps.getProperty("db.url");
				dbUsername = dbProps.getProperty("db.username");
				dbPassword = dbProps.getProperty("db.password");
				String dbTableExistProp = dbProps.getProperty("db.table.exist");

				if ((dbDriver == null) || (dbUrl == null) ||
					(dbUsername == null) || (dbPassword == null) ||
					(dbTableExistProp == null)) {
					
					logger.error("Database properties file corrupted");
					System.exit(1);
				}
				dbTableExist = Integer.parseInt(dbTableExistProp);
			} catch (IOException ex) {
				ex.printStackTrace(System.out);
				System.exit(1);
			} catch (NumberFormatException ex) {
				ex.printStackTrace(System.out);
				System.exit(1);
			}

			logger.info("Database (Driver): " + dbDriver);
			logger.info("Database (URL): " + dbUrl);
			logger.info("Database (Username): " + dbUsername);
			logger.info("Database (Password): " + dbPassword);
			logger.info("Database (Table Already Exist Code): " + dbTableExist);

			Class.forName(dbDriver);
			connection = DriverManager.getConnection(dbUrl, dbUsername, dbPassword);

			statement = connection.createStatement();

			try {
				statement.executeUpdate(CREATE_LOG);
				logger.info("Logging table created");
			} catch (SQLException ex) {
				if (ex.getErrorCode() == dbTableExist)
					logger.warn("Logging Table already existed");
				else
					logger.error(ex.getMessage(), ex);
			}

			try {
				statement.executeUpdate(CREATE_CALLBACK);
				logger.info("Callback table created");
			} catch (SQLException ex) {
				if (ex.getErrorCode() == dbTableExist)
					logger.warn("Callback Table already existed");
				else
					logger.error(ex.getMessage(), ex);
			}

			loggingInsertStatement = connection.prepareStatement(INSERT_LOG);
			loggingQueryStatement = connection.prepareStatement(QUERY_LOG);

			callbackInsertStatement = connection.prepareStatement(INSERT_CALLBACK);
			callbackQueryStatement = connection.prepareStatement(QUERY_CALLBACK);
		} catch (ClassNotFoundException ex) {
			logger.error(ex.getMessage(), ex);
		} catch (SQLException ex) {
			logger.error(ex.getMessage(), ex);
		} finally {
			try {
				if (statement != null)
					statement.close();
			} catch (SQLException ex) {
				logger.error(ex.getMessage(), ex);
			}
		}
	}

	public static DBConnectionPool getInstance(Object context) {
		if (thePool == null) {
			servletContext = ((ServletEndpointContext)context).getServletContext();
			thePool = new DBConnectionPool(servletContext);
		}

		return thePool;
	}

	public synchronized void insert(int table, Vector setters) {
		try {
			if (connection.isClosed()) {
				logger.warn("Initializing the database connection again");
				thePool = new DBConnectionPool(servletContext);
			}
			int rowCount = 0;
			if (table == LOGGING) {
				for (int i=0; i<setters.size()-1; i++)
					loggingInsertStatement.setString(i+1, (String)setters.get(i));
				loggingInsertStatement.setTimestamp(5, (Timestamp)setters.get(4));
				rowCount = loggingInsertStatement.executeUpdate();
				if (logger.isTraceEnabled())
					logger.trace(rowCount + " log row(s) updated");
			} else if (table == CALLBACK) {
				for (int i=0; i<setters.size(); i++)
					callbackInsertStatement.setString(i+1, (String)setters.get(i));
				rowCount = callbackInsertStatement.executeUpdate();
				if (logger.isTraceEnabled())
					logger.trace(rowCount + " callback row(s) updated");
			} else
				logger.error("Unknown table number: \"" + table + "\"");
		} catch (SQLException ex) {
			logger.error(ex.getMessage(), ex);
		}
	}

	public synchronized ArrayList query(int table, Vector setters) {
		ArrayList list = new ArrayList();

		ResultSet rs = null;

		try {
			if (connection.isClosed()) {
				logger.warn("Initializing the database connection again");
				thePool = new DBConnectionPool(servletContext);
			}
			if (table == LOGGING) {

				if (logger.isTraceEnabled())
					logger.trace("Querying LOGGING table");
				for (int i=0; i<setters.size(); i++)
					loggingQueryStatement.setString(i+1, (String)setters.get(i));
				rs = loggingQueryStatement.executeQuery();

				Calendar cal = Calendar.getInstance();

				while (rs.next()) {
					LogEntry logEntry = new LogEntry();
					logEntry.setServiceID(rs.getString(2));
					logEntry.setEventID(rs.getString(3));
					logEntry.setEventDescription(rs.getString(4));
					cal.setTimeInMillis(((Timestamp)rs.getTimestamp(5)).getTime());
					logEntry.setTimestamp(cal);
					list.add(logEntry);
				}
			} else if (table == CALLBACK) {
				if (logger.isTraceEnabled())
					logger.trace("Querying CALLBACK table");
				for (int i=0; i<setters.size(); i++)
					callbackQueryStatement.setString(i+1, (String)setters.get(i));
				rs = callbackQueryStatement.executeQuery();

				while (rs.next()) {
					list.add(rs.getString(1));
					list.add(rs.getString(2));
				}
			} else {
				logger.error("Unknown table number: \"" + table + "\"");
			}
		} catch (SQLException ex) {
			logger.error(ex.getMessage(), ex);
		} finally {
			try {
				if (rs != null)
					rs.close();
			} catch (SQLException ex) {
				logger.error(ex.getMessage(), ex);
			}
		}
		return list;
	}
}

